@@ -21,7 +21,7 @@
 
 #include "rdesktop.h"
 
-#define MAX_CHANNELS			4
+#define MAX_CHANNELS			6
 #define CHANNEL_CHUNK_LENGTH		1600
 #define CHANNEL_FLAG_FIRST		0x01
 #define CHANNEL_FLAG_LAST		0x02
@@ -87,7 +87,7 @@ channel_send(STREAM s, VCHANNEL * channel)
 	s_pop_layer(s, channel_hdr);
 	length = s->end - s->p - 8;
 
-	DEBUG_CLIPBOARD(("channel_send, length = %d\n", length));
+	DEBUG_CHANNEL(("channel_send, length = %d\n", length));
 
 	thislength = MIN(length, CHANNEL_CHUNK_LENGTH);
 /* Note: In the original clipboard implementation, this number was
@@ -102,7 +102,7 @@ channel_send(STREAM s, VCHANNEL * channel)
 	out_uint32_le(s, length);
 	out_uint32_le(s, flags);
 	data = s->end = s->p + thislength;
-	DEBUG_CLIPBOARD(("Sending %d bytes with FLAG_FIRST\n", thislength));
+	DEBUG_CHANNEL(("Sending %d bytes with FLAG_FIRST\n", thislength));
 	sec_send_to_channel(s, g_encryption ? SEC_ENCRYPT : 0, channel->mcs_id);
 
 	/* subsequent segments copied (otherwise would have to generate headers backwards) */
@@ -114,7 +114,7 @@ channel_send(STREAM s, VCHANNEL * channel)
 		if (channel->flags & CHANNEL_OPTION_SHOW_PROTOCOL)
 			flags |= CHANNEL_FLAG_SHOW_PROTOCOL;
 
-		DEBUG_CLIPBOARD(("Sending %d bytes with flags %d\n", thislength, flags));
+		DEBUG_CHANNEL(("Sending %d bytes with flags %d\n", thislength, flags));
 
 		s = sec_init(g_encryption ? SEC_ENCRYPT : 0, thislength + 8);
 		out_uint32_le(s, length);
