@@ -29,12 +29,15 @@
 extern char g_hostname[16];
 extern int g_width;
 extern int g_height;
-extern int g_keylayout;
+extern unsigned int g_keylayout;
+extern int g_keyboard_type;
+extern int g_keyboard_subtype;
+extern int g_keyboard_functionkeys;
 extern BOOL g_encryption;
 extern BOOL g_licence_issued;
 extern BOOL g_use_rdp5;
 extern BOOL g_console_session;
-extern int g_server_bpp;
+extern int g_server_depth;
 extern uint16 mcs_userid;
 extern VCHANNEL g_channels[];
 extern unsigned int g_num_channels;
@@ -43,16 +46,21 @@ static int rc4_key_len;
 static RC4_KEY rc4_decrypt_key;
 static RC4_KEY rc4_encrypt_key;
 static RSA *server_public_key;
+static uint32 server_public_key_len;
 
 static uint8 sec_sign_key[16];
 static uint8 sec_decrypt_key[16];
 static uint8 sec_encrypt_key[16];
 static uint8 sec_decrypt_update_key[16];
 static uint8 sec_encrypt_update_key[16];
-static uint8 sec_crypted_random[SEC_MODULUS_SIZE];
+static uint8 sec_crypted_random[SEC_MAX_MODULUS_SIZE];
 
 uint16 g_server_rdp_version = 0;
 
+/* These values must be available to reset state - Session Directory */
+static int sec_encrypt_use_count = 0;
+static int sec_decrypt_use_count = 0;
+
 /*
  * I believe this is based on SSLv3 with the following differences:
  *  MAC algorithm (5.2.3.1) uses only 32-bit length in place of seq_num/type/length fields
@@ -248,34 +256,30 @@ sec_update(uint8 * key, uint8 * update_key)
 static void
 sec_encrypt(uint8 * data, int length)
 {
-	static int use_count;
-
-	if (use_count == 4096)
+	if (sec_encrypt_use_count == 4096)
 	{
 		sec_update(sec_encrypt_key, sec_encrypt_update_key);
 		RC4_set_key(&rc4_encrypt_key, rc4_key_len, sec_encrypt_key);
-		use_count = 0;
+		sec_encrypt_use_count = 0;
 	}
 
 	RC4(&rc4_encrypt_key, length, data, data);
-	use_count++;
+	sec_encrypt_use_count++;
 }
 
 /* Decrypt data using RC4 */
 void
 sec_decrypt(uint8 * data, int length)
 {
-	static int use_count;
-
-	if (use_count == 4096)
+	if (sec_decrypt_use_count == 4096)
 	{
 		sec_update(sec_decrypt_key, sec_decrypt_update_key);
 		RC4_set_key(&rc4_decrypt_key, rc4_key_len, sec_decrypt_key);
-		use_count = 0;
+		sec_decrypt_use_count = 0;
 	}
 
 	RC4(&rc4_decrypt_key, length, data, data);
-	use_count++;
+	sec_decrypt_use_count++;
 }
 
 static void
@@ -294,14 +298,15 @@ reverse(uint8 * p, int len)
 
 /* Perform an RSA public key encryption operation */
 static void
-sec_rsa_encrypt(uint8 * out, uint8 * in, int len, uint8 * modulus, uint8 * exponent)
+sec_rsa_encrypt(uint8 * out, uint8 * in, int len, uint32 modulus_size, uint8 * modulus,
+		uint8 * exponent)
 {
 	BN_CTX *ctx;
 	BIGNUM mod, exp, x, y;
-	uint8 inr[SEC_MODULUS_SIZE];
+	uint8 inr[SEC_MAX_MODULUS_SIZE];
 	int outlen;
 
-	reverse(modulus, SEC_MODULUS_SIZE);
+	reverse(modulus, modulus_size);
 	reverse(exponent, SEC_EXPONENT_SIZE);
 	memcpy(inr, in, len);
 	reverse(inr, len);
@@ -312,14 +317,14 @@ sec_rsa_encrypt(uint8 * out, uint8 * in, int len, uint8 * modulus, uint8 * expon
 	BN_init(&x);
 	BN_init(&y);
 
-	BN_bin2bn(modulus, SEC_MODULUS_SIZE, &mod);
+	BN_bin2bn(modulus, modulus_size, &mod);
 	BN_bin2bn(exponent, SEC_EXPONENT_SIZE, &exp);
 	BN_bin2bn(inr, len, &x);
 	BN_mod_exp(&y, &x, &exp, &mod, ctx);
 	outlen = BN_bn2bin(&y, out);
 	reverse(out, outlen);
-	if (outlen < SEC_MODULUS_SIZE)
-		memset(out + outlen, 0, SEC_MODULUS_SIZE - outlen);
+	if (outlen < modulus_size)
+		memset(out + outlen, 0, modulus_size - outlen);
 
 	BN_free(&y);
 	BN_clear_free(&x);
@@ -385,14 +390,14 @@ sec_send(STREAM s, uint32 flags)
 static void
 sec_establish_key(void)
 {
-	uint32 length = SEC_MODULUS_SIZE + SEC_PADDING_SIZE;
+	uint32 length = server_public_key_len + SEC_PADDING_SIZE;
 	uint32 flags = SEC_CLIENT_RANDOM;
 	STREAM s;
 
-	s = sec_init(flags, 76);
+	s = sec_init(flags, length + 4);
 
 	out_uint32_le(s, length);
-	out_uint8p(s, sec_crypted_random, SEC_MODULUS_SIZE);
+	out_uint8p(s, sec_crypted_random, server_public_key_len);
 	out_uint8s(s, SEC_PADDING_SIZE);
 
 	s_mark_end(s);
@@ -446,15 +451,17 @@ sec_out_mcs_data(STREAM s)
 	rdp_out_unistr(s, g_hostname, hostlen);
 	out_uint8s(s, 30 - hostlen);
 
-	out_uint32_le(s, 4);
-	out_uint32(s, 0);
-	out_uint32_le(s, 12);
+	/* See
+	   http://msdn.microsoft.com/library/default.asp?url=/library/en-us/wceddk40/html/cxtsksupportingremotedesktopprotocol.asp */
+	out_uint32_le(s, g_keyboard_type);
+	out_uint32_le(s, g_keyboard_subtype);
+	out_uint32_le(s, g_keyboard_functionkeys);
 	out_uint8s(s, 64);	/* reserved? 4 + 12 doublewords */
 	out_uint16_le(s, 0xca01);	/* colour depth? */
 	out_uint16_le(s, 1);
 
 	out_uint32(s, 0);
-	out_uint8(s, g_server_bpp);
+	out_uint8(s, g_server_depth);
 	out_uint16_le(s, 0x0700);
 	out_uint8(s, 0);
 	out_uint32_le(s, 1);
@@ -502,16 +509,18 @@ sec_parse_public_key(STREAM s, uint8 ** modulus, uint8 ** exponent)
 	}
 
 	in_uint32_le(s, modulus_len);
-	if (modulus_len != SEC_MODULUS_SIZE + SEC_PADDING_SIZE)
+	modulus_len -= SEC_PADDING_SIZE;
+	if ((modulus_len < 64) || (modulus_len > SEC_MAX_MODULUS_SIZE))
 	{
-		error("modulus len 0x%x\n", modulus_len);
+		error("Bad server public key size (%u bits)\n", modulus_len * 8);
 		return False;
 	}
 
 	in_uint8s(s, 8);	/* modulus_bits, unknown */
 	in_uint8p(s, *exponent, SEC_EXPONENT_SIZE);
-	in_uint8p(s, *modulus, SEC_MODULUS_SIZE);
+	in_uint8p(s, *modulus, modulus_len);
 	in_uint8s(s, SEC_PADDING_SIZE);
+	server_public_key_len = modulus_len;
 
 	return s_check(s);
 }
@@ -528,6 +537,7 @@ sec_parse_x509_key(X509 * cert)
 	if (OBJ_obj2nid(cert->cert_info->key->algor->algorithm) == NID_md5WithRSAEncryption)
 	{
 		DEBUG_RDP5(("Re-setting algorithm type to RSA in server certificate\n"));
+		ASN1_OBJECT_free(cert->cert_info->key->algor->algorithm);
 		cert->cert_info->key->algor->algorithm = OBJ_nid2obj(NID_rsaEncryption);
 	}
 	epk = X509_get_pubkey(cert);
@@ -537,7 +547,15 @@ sec_parse_x509_key(X509 * cert)
 		return False;
 	}
 
-	server_public_key = (RSA *) epk->pkey.ptr;
+	server_public_key = RSAPublicKey_dup((RSA *) epk->pkey.ptr);
+	EVP_PKEY_free(epk);
+
+	server_public_key_len = RSA_size(server_public_key);
+	if ((server_public_key_len < 64) || (server_public_key_len > SEC_MAX_MODULUS_SIZE))
+	{
+		error("Bad server public key size (%u bits)\n", server_public_key_len * 8);
+		return False;
+	}
 
 	return True;
 }
@@ -675,6 +693,8 @@ sec_parse_crypt_info(STREAM s, uint32 * rc4_key_size,
 		   MITM-attacks.
 		 */
 
+		X509_free(cacert);
+
 		in_uint32_le(s, cert_len);
 		DEBUG_RDP5(("Certificate length is %d\n", cert_len));
 		server_cert = d2i_X509(NULL, &(s->p), cert_len);
@@ -693,8 +713,10 @@ sec_parse_crypt_info(STREAM s, uint32 * rc4_key_size,
 		if (!sec_parse_x509_key(server_cert))
 		{
 			DEBUG_RDP5(("Didn't parse X509 correctly\n"));
+			X509_free(server_cert);
 			return False;
 		}
+		X509_free(server_cert);
 		return True;	/* There's some garbage here we don't care about */
 	}
 	return s_check_end(s);
@@ -704,10 +726,9 @@ sec_parse_crypt_info(STREAM s, uint32 * rc4_key_size,
 static void
 sec_process_crypt_info(STREAM s)
 {
-	uint8 *server_random, *modulus, *exponent;
+	uint8 *server_random, *modulus = NULL, *exponent = NULL;
 	uint8 client_random[SEC_RANDOM_SIZE];
 	uint32 rc4_key_size;
-	uint8 inr[SEC_MODULUS_SIZE];
 
 	if (!sec_parse_crypt_info(s, &rc4_key_size, &server_random, &modulus, &exponent))
 	{
@@ -716,35 +737,39 @@ sec_process_crypt_info(STREAM s)
 	}
 
 	DEBUG(("Generating client random\n"));
-	/* Generate a client random, and hence determine encryption keys */
-	/* This is what the MS client do: */
-	memset(inr, 0, SEC_RANDOM_SIZE);
-	/*  *ARIGL!* Plaintext attack, anyone?
-	   I tried doing:
-	   generate_random(inr);
-	   ..but that generates connection errors now and then (yes, 
-	   "now and then". Something like 0 to 3 attempts needed before a 
-	   successful connection. Nice. Not! 
-	 */
-
 	generate_random(client_random);
+
 	if (NULL != server_public_key)
 	{			/* Which means we should use 
 				   RDP5-style encryption */
+		uint8 inr[SEC_MAX_MODULUS_SIZE];
+		uint32 padding_len = server_public_key_len - SEC_RANDOM_SIZE;
 
-		memcpy(inr + SEC_RANDOM_SIZE, client_random, SEC_RANDOM_SIZE);
-		reverse(inr + SEC_RANDOM_SIZE, SEC_RANDOM_SIZE);
+		/* This is what the MS client do: */
+		memset(inr, 0, padding_len);
+		/*  *ARIGL!* Plaintext attack, anyone?
+		   I tried doing:
+		   generate_random(inr);
+		   ..but that generates connection errors now and then (yes, 
+		   "now and then". Something like 0 to 3 attempts needed before a 
+		   successful connection. Nice. Not! 
+		 */
+		memcpy(inr + padding_len, client_random, SEC_RANDOM_SIZE);
+		reverse(inr + padding_len, SEC_RANDOM_SIZE);
 
-		RSA_public_encrypt(SEC_MODULUS_SIZE,
+		RSA_public_encrypt(server_public_key_len,
 				   inr, sec_crypted_random, server_public_key, RSA_NO_PADDING);
 
-		reverse(sec_crypted_random, SEC_MODULUS_SIZE);
+		reverse(sec_crypted_random, server_public_key_len);
 
+		RSA_free(server_public_key);
+		server_public_key = NULL;
 	}
 	else
 	{			/* RDP4-style encryption */
 		sec_rsa_encrypt(sec_crypted_random,
-				client_random, SEC_RANDOM_SIZE, modulus, exponent);
+				client_random, SEC_RANDOM_SIZE, server_public_key_len, modulus,
+				exponent);
 	}
 	sec_generate_keys(client_random, server_random, rc4_key_size);
 }
@@ -759,7 +784,7 @@ sec_process_srv_info(STREAM s)
 	if (1 == g_server_rdp_version)
 	{
 		g_use_rdp5 = 0;
-		g_server_bpp = 8;
+		g_server_depth = 8;
 	}
 }
 
@@ -848,6 +873,41 @@ sec_recv(uint8 * rdpver)
 				licence_process(s);
 				continue;
 			}
+
+			if (sec_flags & 0x0400)	/* SEC_REDIRECT_ENCRYPT */
+			{
+				uint8 swapbyte;
+
+				in_uint8s(s, 8);	/* signature */
+				sec_decrypt(s->p, s->end - s->p);
+
+				/* Check for a redirect packet, starts with 00 04 */
+				if (s->p[0] == 0 && s->p[1] == 4)
+				{
+					/* for some reason the PDU and the length seem to be swapped.
+					   This isn't good, but we're going to do a byte for byte
+					   swap.  So the first foure value appear as: 00 04 XX YY,
+					   where XX YY is the little endian length. We're going to
+					   use 04 00 as the PDU type, so after our swap this will look
+					   like: XX YY 04 00 */
+					swapbyte = s->p[0];
+					s->p[0] = s->p[2];
+					s->p[2] = swapbyte;
+
+					swapbyte = s->p[1];
+					s->p[1] = s->p[3];
+					s->p[3] = swapbyte;
+
+					swapbyte = s->p[2];
+					s->p[2] = s->p[3];
+					s->p[3] = swapbyte;
+				}
+#ifdef WITH_DEBUG
+				/* warning!  this debug statement will show passwords in the clear! */
+				hexdump(s->p, s->end - s->p);
+#endif
+			}
+
 		}
 
 		if (channel != MCS_GLOBAL_CHANNEL)
@@ -884,9 +944,40 @@ sec_connect(char *server, char *username)
 	return True;
 }
 
+/* Establish a secure connection */
+BOOL
+sec_reconnect(char *server)
+{
+	struct stream mcs_data;
+
+	/* We exchange some RDP data during the MCS-Connect */
+	mcs_data.size = 512;
+	mcs_data.p = mcs_data.data = (uint8 *) xmalloc(mcs_data.size);
+	sec_out_mcs_data(&mcs_data);
+
+	if (!mcs_reconnect(server, &mcs_data))
+		return False;
+
+	/*      sec_process_mcs_data(&mcs_data); */
+	if (g_encryption)
+		sec_establish_key();
+	xfree(mcs_data.data);
+	return True;
+}
+
 /* Disconnect a connection */
 void
 sec_disconnect(void)
 {
 	mcs_disconnect();
 }
+
+/* reset the state of the sec layer */
+void
+sec_reset_state(void)
+{
+	g_server_rdp_version = 0;
+	sec_encrypt_use_count = 0;
+	sec_decrypt_use_count = 0;
+	mcs_reset_state();
+}
